# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: Philip J Grabner <grabner@cadit.com>
# date: 2013/07/10
# copy: (C) Copyright 2013 Cadit Inc., All Rights Reserved.
#------------------------------------------------------------------------------

import sys, unittest, re
import templatealchemy as ta

from .manager import Manager
from .sender import Sender, StoredSender
from . import util

def eqstrip(val):
  val = val.replace('\r\n', '\n').replace('\r', '\n')
  val = re.sub('[ \t]+', ' ', val)
  return val.strip()

def wsstrip(val):
  return re.sub('\s+', '', val)

def template(s, renderer='mako'):
  return ta.Template(source='string:' + s, renderer=renderer)

#------------------------------------------------------------------------------
class TestEmail(unittest.TestCase):

  maxDiff = None

  #----------------------------------------------------------------------------
  def assertXmlEqual(self, val, chk):
    if val == chk:
      return self.assertEqual(val, chk)
    if wsstrip(val) == wsstrip(chk):
      return self.assertEqual(wsstrip(val), wsstrip(chk))
    # TODO: make this XML-normalize the values first...
    self.assertMultiLineEqual(eqstrip(val),eqstrip(chk))

  #----------------------------------------------------------------------------
  def assertMimeXmlEqual(self, val, chk):
    if val == chk:
      return self.assertEqual(val, chk)
    if wsstrip(val) == wsstrip(chk):
      return self.assertEqual(wsstrip(val), wsstrip(chk))
    # TODO: make this XML-normalize the XML/HTML values first...
    self.assertMultiLineEqual(eqstrip(val), eqstrip(chk))

  #----------------------------------------------------------------------------
  def test_inlineHtmlStyling(self):
    html = '<html><body><div>foo</div></body></html>'
    css  = 'body{color:red}body > div{font-size:10px}'
    chk  = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <body style="color: red">
    <div style="font-size: 10px">foo</div>
  </body>
</html>
'''
    xdoc = util.parseXml(html)
    xdoc = util.inlineHtmlStyling(xdoc, css)
    out  = util.serializeHtml(xdoc)
    self.assertXmlEqual(out, chk)

  #----------------------------------------------------------------------------
  def test_deprecationwarning_workaround(self):
    html = '<html><body><p class="f">first</p><p>second</p></body></html>'
    css  = 'p{background:#666;}p.f{background:#042d5a;}'
    import warnings
    warnings.filterwarnings(
      "error",
      message='Call to deprecated method \'_[sg]etCSSValue\'. Use ``property.propertyValue`` instead.',
      category=DeprecationWarning,
      module='cssutils.css.cssstyledeclaration',
      lineno=598,
      )
    xdoc = util.parseXml(html)
    xdoc = util.inlineHtmlStyling(xdoc, css)
    out  = util.serializeHtml(xdoc)
    self.assertXmlEqual('''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <body>
    <p class="f" style="background: #042d5a">first</p>
    <p style="background: #666">second</p>
  </body>
</html>
''',
                     out)

  #----------------------------------------------------------------------------
  def test_reduce2ascii_is_needed(self):
    from .util import reduce2ascii as r2a
    self.assertRaises(UnicodeEncodeError,
                      u'this \u21d2 that'.encode,
                      'ascii')
    r2a(u'this \u21d2 that').encode('ascii')

  #----------------------------------------------------------------------------
  def test_reduce2ascii(self):
    from .util import reduce2ascii as r2a
    self.assertEqual('this => that', r2a(u'this \u21d2 that'))
    self.assertEqual('this <- that', r2a(u'this \u2190 that'))
    self.assertEqual('this <= that', r2a(u'this \u21e6 that'))
    self.assertEqual(
      '->=>->=>=>->|->->|->=>>->|=>~>->>->',
      r2a(u'\u2192\u21d2\u21ac\u21c9\u21e8\u21e5\u21e2\u21aa\u21a6\u21f6\u21a3\u21f0\u219d\u21a0\u21fe'))

  #----------------------------------------------------------------------------
  def test_callingPkgName(self):
    from templatealchemy.util import callingPkgName
    self.assertEqual(callingPkgName(), 'genemail')
    self.assertEqual(callingPkgName(ignore='genemail'), 'unittest')

  #----------------------------------------------------------------------------
  def test_subject_element(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title><email:subject share="content">${message.title()}</email:subject></title>
 </head>
 <body>
  <p>${message}.</p>
 </body>
</html>
'''
    chk = 'This Is A Test'
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    eml['message'] = 'This is a test'
    self.assertEqual(chk, eml.getSubject())
    # TODO: ensure share="content" works... (it doesn't currently...)

  #----------------------------------------------------------------------------
  def test_subject_entities(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title email:subject="content">${message.title()}&nbsp;&mdash;&nbsp;And More!...</title>
 </head>
 <body>
  <h1>Foo &amp; Bar</h1>
  <p>${message}&nbsp;&mdash;&nbsp;and more!...</p>
 </body>
</html>
'''
    schk = 'This & That -- And More!...'
    hchk = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>This &amp; That&#160;&#8212;&#160;And More!...</title>
  </head>
  <body>
  <h1>Foo &amp; Bar</h1>
  <p>This &amp; that&#160;&#8212;&#160;and more!...</p>
 </body>
</html>
'''
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    eml['message'] = 'This & that'
    self.assertEqual(schk, eml.getSubject())
    self.assertXmlEqual(hchk, eml.getHtml())

  #----------------------------------------------------------------------------
  def test_subject_attribute(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title email:subject="content">${message.title()}</title>
 </head>
 <body>
  <p>${message}.</p>
 </body>
</html>
'''
    chk = 'This Is A Test'
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    eml['message'] = 'This is a test'
    self.assertEqual(chk, eml.getSubject())

  #----------------------------------------------------------------------------
  def test_subject_maxlength(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title><email:subject share="content">${message.title()}</email:subject></title>
 </head>
 <body>
  <p>${message}.</p>
 </body>
</html>
'''
    chk = 'This Is A Test [...]'
    man = Manager(sender=StoredSender(), provider=template(tpl))
    man.default.maxSubjectLength = 20
    eml = man.newEmail()
    eml['message'] = 'This is a test of capping the subject length'
    self.assertEqual(chk, eml.getSubject())
    chk2 = 'This [...]'
    eml2 = man.newEmail()
    eml2.maxSubjectLength = 10
    eml2['message'] = 'This is a test of capping the subject length'
    self.assertEqual(chk2, eml2.getSubject())

  #----------------------------------------------------------------------------
  def test_subject_snip(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title><email:subject share="content">${message.title()}</email:subject></title>
 </head>
 <body>
  <p>${message}.</p>
 </body>
</html>
'''
    chk = 'This Is [...snip...]'
    man = Manager(sender=StoredSender(), provider=template(tpl))
    man.default.maxSubjectLength = 20
    man.default.snipIndicator = '[...snip...]'
    eml = man.newEmail()
    eml['message'] = 'This is a test of changing the snip indicator'
    self.assertEqual(chk, eml.getSubject())

  #----------------------------------------------------------------------------
  def test_simpleEmail_textOnly(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From" value="noreply@example.com">this-is-bogus</email:header>
  <email:env name="unused">feedback value</email:env>
  <title email:subject="content">${message.title()}</title>
 </head>
 <body>
  <p>${message}.</p>
  <p>Also sent to: <span email:header="CC">foo@example.com</span>.</p>
 </body>
</html>
'''
    tchk = '''\
This is a test.

Also sent to: foo@example.com.
'''
    etchk = '''\
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit
From: noreply@example.com
CC: foo@example.com
To: test@example.com
Date: Fri, 13 Feb 2009 23:31:30 -0000
Message-ID: <1234567890@genemail.example.com>
Subject: This Is A Test\n\n''' + tchk
    hchk = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>This Is A Test</title>
  </head>
  <body>
  <p>This is a test.</p>
  <p>Also sent to: <span>foo@example.com</span>.</p>
 </body>
</html>
'''
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    # override the UN-predictable generated info...
    eml.setHeader('Date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('Message-ID', '<1234567890@genemail.example.com>')
    eml.includeComponents = ['text']
    eml['message'] = 'This is a test'
    eml.send()
    self.assertEqual(1, len(eml.manager.sender.emails))
    out = eml.manager.sender.emails[0]
    self.assertMultiLineEqual(tchk, eml.getText())
    self.assertXmlEqual(hchk, eml.getHtml())
    self.assertMultiLineEqual(etchk, out['message'])
    self.assertEqual(sorted(['test@example.com', 'foo@example.com']),
                     sorted(out['recipients']))

  #----------------------------------------------------------------------------
  def test_cidCleanup(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From" value="noreply@example.com">this-is-bogus</email:header>
  <title email:subject="content">${message.title()}</title>
  <email:attachment name="smiley.png" encoding="base64" cid="true">
   dGhpcyBpcyBhIGJvZ3VzIGltYWdlCg==
  </email:attachment>
 </head>
 <body>
  <p>${message}.</p>
  <p>Also sent to: <span email:header="CC">foo@example.com</span>.</p>
  <p><img alt="smiley" src="cid:smiley.png"/></p>
 </body>
</html>
'''
    chk = '''\
This is a test.

Also sent to: foo@example.com.

[smiley]
'''
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    eml['message'] = 'This is a test'
    self.assertMultiLineEqual(chk, eml.getText())

  #----------------------------------------------------------------------------
  def test_inlineStyling(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From" value="noreply@example.com">this-is-bogus</email:header>
  <title email:subject="content">${message.title()}</title>
<style type="text/css">
p > span { font-weight: bold; }
  </style>
 </head>
 <body>
  <p>${message}.</p>
  <p>Also sent to: <span email:header="CC">foo@example.com</span>.</p>
 </body>
</html>
'''
    tchk = '''\
This is a test.

Also sent to: foo@example.com.
'''
    etchk = '''\
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit
From: noreply@example.com
CC: foo@example.com
To: test@example.com
Date: Fri, 13 Feb 2009 23:31:30 -0000
Message-ID: <1234567890@genemail.example.com>
Subject: This Is A Test\n\n''' + tchk
    hchk = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>This Is A Test</title>
  </head>
  <body>
  <p>This is a test.</p>
  <p>Also sent to: <span style="font-weight: bold">foo@example.com</span>.</p>
 </body>
</html>
'''
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    # override the UN-predictable generated info...
    eml.setHeader('Date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('Message-ID', '<1234567890@genemail.example.com>')
    eml.includeComponents = ['text']
    eml['message'] = 'This is a test'
    eml.send()
    #self.assertMultiLineEqual(etchk, eml.manager.sender.emails[0]['message'])
    #self.assertMultiLineEqual(tchk, eml.getText())
    self.assertXmlEqual(hchk, eml.getHtml())

  #----------------------------------------------------------------------------
  def test_email_rawSmtp(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From" value="noreply@example.com"/>
  <email:attachment name="smiley.png" encoding="base64" cid="true">
   iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAACXBIWXMAAAsTAAALEwEAmpwYAAAA
   IGNIUk0AAHolAACAgwAA+f8AAIDpAAB1MAAA6mAAADqYAAAXb5JfxUYAAAK0SURBVHjabJNfaJNX
   GMZ/X/IlaWub1NqWzLY6dYyiKAWxF0MdyC4KgjeObWy72G28EESam93sOlUZA82Nt2OMiWWwQmFg
   BXfVIQ6JqdY0sRi1TZfG/OmX7/vOOXm9KIRU+8KB9+F9zvPAe85jiQidtfrnxJCIJIAphEkBEFkU
   mEdIH774eKOTb3UKFO6emA6E+1LR0c/pHjxGV+wQAG61wNZGhurqAtqvJz/96snMBwK5O8dvxcbO
   Jvo//oJQpBdaLiJqm2SFINCFcuuU83+zWVhIH/3u6aW2wPLvx1KxsTPTw+MXQFdA/Y/oGrScbZtA
   D5YdhdAg2Ht5nZmlnL8/c+KH50kr++v4SDASKx757DKWWgPvFYv35kE0p05PAPDvP/+BZTN5bgoi
   I4gdJ7twDd+pjAa0NolYfALLySO1DNIosJJdZmUpD34T/CYrS3lWsstIo4DUMljNPAMjJ9HKJGyt
   zPk9PV20qllQFRDTXqr47o4XEq8BfhPxXfp6h1DKnLe1Nkcjpoy4b3ZcBhDl7o6VSyQEWptDtlKm
   qRtvwsHWVpv4zdXbAJjczV0xQEtvoJQJ2lqZQrNentgTbrWHeunGDuf3MYDjltDK5AJa6bnN8ltE
   +YjyWXz0st2/fzpnpVIFrcxcQCmTfrnewPguaI8DBw7y4GGRtfUqnuPgOQ5r61UePCwyFAuB9jC+
   y4vXVbQ2aUtE+Oun/tTIgD09vr8Lb+9ZNktFni0/p16vAdDXF+XIaIyP4kPYzQKZ1S1W192ZL1PN
   ZPsrz/7Ye2tsIJQ4vL+f7micQPQTLLt3e/u6QauWw6kUyb2q8qLkpb++7l36IEx/JLunw0FSB/fZ
   DEeD9PcEAKhsGUo1Q2FD4fqt5Lc/q5ld0wjw25VwXCCByJTAJNt5XhRhHkh//4te6+S/GwD2npI7
   ZK15EgAAAABJRU5ErkJggg==
  </email:attachment>
  <title email:subject="content">${message.title()}</title>
 </head>
 <body>
  <p>${message}.</p>
  <p><img alt="smiley" src="cid:smiley.png"/></p>
 </body>
</html>
'''
    schk = 'This And That'
    hchk = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>This And That</title>
  </head>
  <body>
  <p>This and that.</p>
  <p><img alt="smiley" src="cid:smiley.png"/></p>
 </body>
</html>
'''

    regex = re.compile('.*<email:attachment[^>]*>([^<]*)</email:attachment>.*', re.DOTALL)

    chk = '''\
Content-Type: multipart/alternative; boundary="==genemail.test-alt-2=="
MIME-Version: 1.0
Date: Fri, 13 Feb 2009 23:31:30 -0000
To: test@example.com
Message-ID: <1234567890@genemail.example.com>
From: noreply@example.com
Subject: This And That

--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit

This and that.

[smiley]


--==genemail.test-alt-2==
Content-Type: multipart/related; boundary="==genemail.test-rel-3=="
MIME-Version: 1.0

--==genemail.test-rel-3==
MIME-Version: 1.0
Content-Type: text/html; charset="us-ascii"
Content-Transfer-Encoding: 7bit

<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>This And That</title>
  </head>
  <body>
  <p>This and that.</p>
  <p><img alt="smiley" src="cid:smiley.png"/></p>
 </body>
</html>

--==genemail.test-rel-3==
Content-Type: image/png; name="smiley.png"
MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Disposition: attachment
Content-ID: <smiley.png>

''' + regex.sub('\\1', tpl).replace('   ','').strip() + '''
--==genemail.test-rel-3==--
--==genemail.test-alt-2==--
'''

    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    # override the UNpredictable generated info...
    eml.setHeader('Date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('Message-ID', '<1234567890@genemail.example.com>')
    eml.boundary = 'genemail.test'
    eml['message'] = 'This and that'
    eml.send()
    self.assertEqual(schk, eml.getSubject())
    self.assertXmlEqual(hchk, eml.getHtml())
    self.assertMimeXmlEqual(eml.manager.sender.emails[0]['message'], chk)

  #----------------------------------------------------------------------------
  def test_default_recycling(self):
    tpl1 = '<p xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0">T1 to <span email:header="CC">foo1@example.com</span>.</p>'
    man  = Manager(sender=StoredSender(), provider=template(tpl1))
    eml1 = man.newEmail()
    eml1.setHeader('To', 'test@example.com')
    eml1.setHeader('From', 'noreply@example.com')
    eml1.setHeader('Bcc', 'bcc@example.com')
    eml1.setHeader('Date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml1.setHeader('Message-ID', '<1234567890@genemail.example.com>')
    eml1.includeComponents = ['text']
    eml1.send()
    self.assertEqual(1, len(eml1.manager.sender.emails))
    out1 = eml1.manager.sender.emails[-1]
    self.assertEqual(sorted(['test@example.com', 'foo1@example.com', 'bcc@example.com']),
                     sorted(out1['recipients']))
    eml2 = man.newEmail()
    eml2.setHeader('To', 'test@example.com')
    eml2.setHeader('From', 'noreply@example.com')
    eml2.setHeader('Date', 'Fri, 23 Feb 2009 23:32:30 -0000')
    eml2.setHeader('Message-ID', '<2234567890@genemail.example.com>')
    eml2.includeComponents = ['text']
    eml2.send()
    self.assertEqual(2, len(eml2.manager.sender.emails))
    out2 = eml2.manager.sender.emails[-1]
    self.assertEqual(sorted(['test@example.com', 'foo1@example.com']),
                     sorted(out2['recipients']))

  #----------------------------------------------------------------------------
  def test_bccHeader(self):
    tpl = '''\
<html
 lang="en"
 xml:lang="en"
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title email:subject="content">${message.title()}</title>
  <email:header name="to">rcpt@example.com</email:header>
  <email:header name="from">mailfrom@example.com</email:header>
 </head>
 <body><p>${message}.</p></body>
</html>
'''
    sender  = StoredSender()
    manager = Manager(sender=sender, provider=template(tpl))
    eml = manager.newEmail()
    eml['message'] = 'test'
    eml.setHeader('BcC', 'bcc@example.com')
    # override the UNpredictable generated info...
    eml.setHeader('date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('message-id', '<1234567890@@genemail.example.com>')
    eml.boundary = 'genemail.test'
    eml.send()
    self.assertEqual(1, len(sender.emails))
    out = sender.emails[0]
    self.assertEqual(sorted(['mailfrom', 'recipients', 'message']), sorted(out.keys()))
    self.assertEqual('mailfrom@example.com', out['mailfrom'])
    self.assertEqual(sorted(['rcpt@example.com', 'bcc@example.com']), sorted(out['recipients']))

    # ensure that the 'bcc' headers is stripped out of the output
    chk = '''\
Content-Type: multipart/alternative; boundary="==genemail.test-alt-2=="
MIME-Version: 1.0
From: mailfrom@example.com
To: rcpt@example.com
Date: Fri, 13 Feb 2009 23:31:30 -0000
Message-ID: <1234567890@@genemail.example.com>
Subject: Test

--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit

test.


--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/html; charset="us-ascii"
Content-Transfer-Encoding: 7bit

<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
  <head>
    <title>Test</title>
  </head>
  <body>
    <p>test.</p>
  </body>
</html>

--==genemail.test-alt-2==--
'''
    self.assertMimeXmlEqual(out['message'], chk)

  #----------------------------------------------------------------------------
  def test_noMinimalMime(self):
    tpl = '''\
<html
 lang="en"
 xml:lang="en"
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title email:subject="content">${message.title()}</title>
  <email:header name="to">rcpt@example.com</email:header>
  <email:header name="from">mailfrom@example.com</email:header>
 </head>
 <body><p>${message}.</p></body>
</html>
'''
    manager = Manager(sender=StoredSender(), provider=template(tpl))
    manager.default.minimalMime = False
    eml = manager.newEmail()
    eml['message'] = 'test'
    eml.setHeader('BcC', 'bcc@example.com')
    # override the UNpredictable generated info...
    eml.setHeader('date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('message-id', '<1234567890@@genemail.example.com>')
    eml.boundary = 'genemail.test'
    eml.send()
    self.assertEqual(1, len(manager.sender.emails))
    out = manager.sender.emails[0]
    self.assertEqual(sorted(['mailfrom', 'recipients', 'message']), sorted(out.keys()))
    self.assertEqual('mailfrom@example.com', out['mailfrom'])
    self.assertEqual(sorted(['rcpt@example.com', 'bcc@example.com']), sorted(out['recipients']))
    chk = '''\
Content-Type: multipart/mixed; boundary="==genemail.test-mix-1=="
MIME-Version: 1.0
From: mailfrom@example.com
To: rcpt@example.com
Date: Fri, 13 Feb 2009 23:31:30 -0000
Message-ID: <1234567890@@genemail.example.com>
Subject: Test

--==genemail.test-mix-1==
Content-Type: multipart/alternative; boundary="==genemail.test-alt-2=="
MIME-Version: 1.0

--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit

test.


--==genemail.test-alt-2==
Content-Type: multipart/related; boundary="==genemail.test-rel-3=="
MIME-Version: 1.0

--==genemail.test-rel-3==
MIME-Version: 1.0
Content-Type: text/html; charset="us-ascii"
Content-Transfer-Encoding: 7bit

<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
  <head>
    <title>Test</title>
  </head>
  <body>
    <p>test.</p>
  </body>
</html>

--==genemail.test-rel-3==--
--==genemail.test-alt-2==--
--==genemail.test-mix-1==--
'''
    self.assertMimeXmlEqual(out['message'], chk)

  #----------------------------------------------------------------------------
  def test_managerBccHeader(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <title email:subject="content">${message.title()}</title>
  <email:header name="to">rcpt@example.com</email:header>
  <email:header name="from">mailfrom@example.com</email:header>
 </head>
 <body><p>${message}.</p></body>
</html>
'''
    sender  = StoredSender()
    class MyManager(Manager):
      def updateHeaders(self, emailobj, headers):
        #super(MyManager, self).updateHeaders(emailobj, headers)
        headers['bcc'] = 'bcc@example.com'
    manager = MyManager(sender=sender, provider=template(tpl))
    eml = manager.newEmail()
    eml['message'] = 'test'
    # override the UNpredictable generated info...
    eml.setHeader('date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('message-id', '<1234567890@@genemail.example.com>')
    eml.includeComponents = ['text']
    eml.send()
    self.assertEqual(1, len(sender.emails))
    out = sender.emails[0]
    self.assertEqual(sorted(['rcpt@example.com', 'bcc@example.com']), sorted(out['recipients']))

  # #----------------------------------------------------------------------------
  # def test_multiBccHeader(self):
  #   # todo
  #   pass

  # TODO: test non-CID attachments...
  # TODO: test CID-cleansing...

  #----------------------------------------------------------------------------
  def test_env_extraction(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From" value="noreply@example.com">this-is-bogus</email:header>
  % comment:
-*- spec -*-
settings:
  env-name: feedback env-value
-*- /spec -*-
  % endcomment
  <email:env name="env-name">feedback env-value</email:env>
  <title email:subject="content">${message.title()}</title>
 </head>
 <body>
  <p>${message}.</p>
  <p>Also sent to: <span email:header="CC">foo@example.com</span>.</p>
 </body>
</html>
'''
    eml = Manager(sender=StoredSender(), provider=template(tpl)).newEmail()
    self.assertEqual(eml.getSetting('env-name'), 'feedback env-value')
    self.assertEqual(eml.getSettings(), dict([['env-name', 'feedback env-value']]))

  #----------------------------------------------------------------------------
  def test_standaloneHtml(self):
    tpl = '''\
<html
 xmlns="http://www.w3.org/1999/xhtml"
 xmlns:email="http://pythonhosted.org/genemail/xmlns/1.0"
 >
 <head>
  <email:header name="To">test@example.com</email:header>
  <email:header name="From">noreply@example.com</email:header>
  <title email:subject="content">${message.title()}</title>
  <email:attachment name="slogan.txt" encoding="base64" cid="true">
    QUxMIFlPVVIgQkFTRSBBUkUgQkVMT05HIFRPIFVT
  </email:attachment>
 </head>
 <body>
  <p>${message} <img src="cid:slogan.txt"/>.</p>
 </body>
</html>
'''
    chk = '''\
'''
    manager = Manager(sender=StoredSender(), provider=template(tpl))
    eml = manager.newEmail()
    eml['message'] = 'Foo the bar'
    # override the UNpredictable generated info...
    eml.setHeader('date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('message-id', '<1234567890@@genemail.example.com>')
    eml.boundary = 'genemail.test'
    eml.send()
    self.assertEqual(1, len(manager.sender.emails))
    out = manager.sender.emails[0]
    chk = '''\
Content-Type: multipart/alternative; boundary="==genemail.test-alt-2=="
MIME-Version: 1.0
Date: Fri, 13 Feb 2009 23:31:30 -0000
To: test@example.com
Message-ID: <1234567890@@genemail.example.com>
From: noreply@example.com
Subject: Foo The Bar

--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit

Foo the bar [].

--==genemail.test-alt-2==
Content-Type: multipart/related; boundary="==genemail.test-rel-3=="
MIME-Version: 1.0

--==genemail.test-rel-3==
MIME-Version: 1.0
Content-Type: text/html; charset="us-ascii"
Content-Transfer-Encoding: 7bit

<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Foo The Bar</title>
  </head>
  <body>
    <p>Foo the bar <img src="cid:slogan.txt" />.</p>
  </body>
</html>
--==genemail.test-rel-3==
Content-Type: text/plain; charset="us-ascii"
MIME-Version: 1.0
Content-Transfer-Encoding: 7bit
Content-Disposition: attachment
Content-ID: <slogan.txt>

ALL YOUR BASE ARE BELONG TO US
--==genemail.test-rel-3==--
--==genemail.test-alt-2==--
'''
    self.assertMimeXmlEqual(out['message'], chk)
    chk = '''\
<html xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <title>Foo The Bar</title>
  </head>
  <body>
    <p>Foo the bar <img src="data:text/plain;base64,QUxMIFlPVVIgQkFTRSBBUkUgQkVMT05HIFRPIFVT" />.</p>
  </body>
</html>
'''
    self.assertXmlEqual(chk, eml.getHtml(standalone=True))

  #----------------------------------------------------------------------------
  def test_spec_override(self):
    manager = Manager(
      sender   = StoredSender(),
      provider = ta.Manager(
        source   = 'pkg:genemail:test_data/templates/email',
        renderer = 'mako'),
      # modifier = genemail.DkimModifier(
      #   selector = 'selector._domainkey.example.com',
      #   key      = '/path/to/private-rsa.key',
      #   )
      )
    eml = manager.newEmail('invite')
    eml['name'] = 'Joe Schmoe'
    eml['email'] = 'test@example.com'
    eml.addAttachment(
      name        = 'invite.ics',
      value       = '''\
BEGIN:VCALENDAR
PRODID:-//Mozilla.org/NONSGML Mozilla Calendar V1.1//EN
VERSION:2.0
BEGIN:VEVENT
SUMMARY:Test Invite
END:VEVENT
END:VCALENDAR
''',
      contentType = 'text/calendar; name=invite.ics; method=PUBLISH')
    # override the UNpredictable generated info...
    eml.setHeader('date', 'Fri, 13 Feb 2009 23:31:30 -0000')
    eml.setHeader('message-id', '<1234567890@@genemail.example.com>')
    eml.boundary = 'genemail.test'
    eml.send()
    self.assertEqual(1, len(manager.sender.emails))
    out = manager.sender.emails[0]
    chk = '''\
Content-Type: multipart/mixed; boundary="==genemail.test-mix-1=="
MIME-Version: 1.0
Date: Fri, 13 Feb 2009 23:31:30 -0000
To: "Joe Schmoe" <test@example.com>
Message-ID: <1234567890@@genemail.example.com>
From: "Hello World" <noreply@example.com>
Subject: Hello!

--==genemail.test-mix-1==
Content-Type: multipart/alternative; boundary="==genemail.test-alt-2=="
MIME-Version: 1.0

--==genemail.test-alt-2==
MIME-Version: 1.0
Content-Type: text/plain; charset="us-ascii"
Content-Transfer-Encoding: 7bit

Hello Joe Schmoe, please find your invite attached.

--==genemail.test-alt-2==
Content-Type: multipart/related; boundary="==genemail.test-rel-3=="
MIME-Version: 1.0

--==genemail.test-rel-3==
MIME-Version: 1.0
Content-Type: text/html; charset="us-ascii"
Content-Transfer-Encoding: 7bit

<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
  <head>
    <meta charset="utf-8" />
    <title>Hello!</title>
  </head>
  <body>
    <img src="cid:logo.png" />
    <h1>Hello Joe Schmoe!</h1>
    <p>Please find your invite attached. <img src="cid:smiley.png" /></p>
    <img src="cid:sig.png" />
  </body>
  </html>
--==genemail.test-rel-3==
Content-Type: image/png; name="smiley.png"
MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Disposition: attachment
Content-ID: <smiley.png>

iVBORw0KGgoAAAANSUhEUgAAAAkAAAAJCAQAAABKmM6bAAAAOElEQVQI12WOQQ4AQAQDpxv//3L3
YJvI6oGgA2E+FVizIZ9OciIUxBf3YemB2TPALhqtns+r2n9d0gAQF+Ohrm4AAAAASUVORK5CYII=
--==genemail.test-rel-3==
Content-Type: image/png; name="logo.png"
MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Disposition: attachment
Content-ID: <logo.png>

iVBORw0KGgoAAAANSUhEUgAAAAkAAAAJAQMAAADaX5RTAAAABlBMVEX///8AAABVwtN+AAAAAXRS
TlMAQObYZgAAAAFiS0dEAIgFHUgAAAAgSURBVAjXY6hnYPjPwLCEgUEFjFIYGNIaGJ43MBxmAABI
GwXfd8ROSAAAAABJRU5ErkJggg==
--==genemail.test-rel-3==
Content-Type: image/png; name="sig.png"
MIME-Version: 1.0
Content-Transfer-Encoding: base64
Content-Disposition: attachment
Content-ID: <sig.png>

iVBORw0KGgoAAAANSUhEUgAAAAkAAAAJAQMAAADaX5RTAAAABlBMVEUAAAAAAAClZ7nPAAAAAXRS
TlMAQObYZgAAAB5JREFUCNdj+MDAIMDAIMTAwMjA8L8BRAqBRT4wAAAyoQOmAXy5oQAAAABJRU5E
rkJggg==
--==genemail.test-rel-3==--
--==genemail.test-alt-2==--
--==genemail.test-mix-1==
MIME-Version: 1.0
Content-Type: text/calendar; name="invite.ics"; method="PUBLISH";
 charset="us-ascii"
Content-Transfer-Encoding: 7bit
Content-Disposition: attachment; filename="invite.ics"

BEGIN:VCALENDAR
PRODID:-//Mozilla.org/NONSGML Mozilla Calendar V1.1//EN
VERSION:2.0
BEGIN:VEVENT
SUMMARY:Test Invite
END:VEVENT
END:VCALENDAR

--==genemail.test-mix-1==--
'''
    self.assertMimeXmlEqual(out['message'], chk)

#------------------------------------------------------------------------------
if __name__ == '__main__':
  unittest.main() # pragma: no cover

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# lib:  genemail.util
# auth: Philip J Grabner <grabner@cadit.com>
# date: 2013/07/09
# copy: (C) Copyright 2013 Cadit Health Inc., All Rights Reserved.
#------------------------------------------------------------------------------

from __future__ import absolute_import

import re, codecs
from StringIO import StringIO
import xml.etree.ElementTree as ET
from xml.dom import minidom
import cssutils, cssselect, xpath

#------------------------------------------------------------------------------
htmlns = 'http://www.w3.org/1999/xhtml'

#------------------------------------------------------------------------------
# email regex from:
#   http://www.regular-expressions.info/email.html
# and extended to support "...@localhost"
def extractEmails(s):
  if not s:
    return None
  emailRegex = '\\b[A-Z0-9._%+-]+@(?:(?:[A-Z0-9-]+\\.)+[A-Z]{2,4}|localhost)\\b'
  return re.findall(emailRegex, s, re.I)

#------------------------------------------------------------------------------
def getHtmlStyleView(document, css, media='all', name=None,
                     styleCallback=lambda element: None):
  """
  :param document:
    a DOM element (must be minidom-compatible)
  :param css:
    a CSS StyleSheet string
  :param media:
    [optional] TODO: view for which media it should be
  :param name:
    [optional] TODO: names of sheets only
  :param styleCallback:
    [optional] should return css.CSSStyleDeclaration of inline styles,
    for html a style declaration for ``element@style``. Gets one
    parameter ``element`` which is the relevant DOMElement

  returns style view
    a dict of {DOMElement: css.CSSStyleDeclaration} for html

  shamelessly scrubbed from:
    http://cssutils.googlecode.com/svn/trunk/examples/style.py (``getView()``)
    ==> with an adjustment to get around a deprecationwarning in
        cssutils/css/cssstyledeclaration.py:598
    ==> and an adjustment to use "xhtml" translator for the cssselector
    ==> and an adjustment to not use lxml
  """

  sheet = cssutils.parseString(css)
  css2xpath = cssselect.GenericTranslator()
  view = {}
  specificities = {} # needed temporarily
  # TODO: filter rules simpler?, add @media
  rules = (rule for rule in sheet if rule.type == rule.STYLE_RULE)
  for rule in rules:
    for selector in rule.selectorList:
      xpe = css2xpath.css_to_xpath(selector.selectorText)
      for element in xpath.find(xpe, document):
        if element not in view:
          # add initial empty style declatation
          view[element] = cssutils.css.CSSStyleDeclaration()
          specificities[element] = {}
          # and add inline @style if present
          inlinestyle = styleCallback(element)
          if inlinestyle:
            for p in inlinestyle:
              # set inline style specificity
              view[element].setProperty(p)
              specificities[element][p.name] = (1,0,0,0)
        for p in rule.style:
          # update style declaration
          if p not in view[element]:
            # setProperty needs a new Property object and
            # MUST NOT reuse the existing Property
            # which would be the same for all elements!
            # see Issue #23
            view[element].setProperty(p.name, p.value, p.priority)
            specificities[element][p.name] = selector.specificity
          else:
            sameprio = (p.priority ==
                  view[element].getPropertyPriority(p.name))
            if not sameprio and bool(p.priority) or (
               sameprio and selector.specificity >=
                    specificities[element][p.name]):
              # later, more specific or higher prio
              # NOTE: added explicit removeProperty to get around these warnings:
              #   cssutils-0.9.8a1-py2.6.egg/cssutils/css/cssstyledeclaration.py:598: DeprecationWarning: Call to deprecated method '_getCSSValue'. Use ``property.propertyValue`` instead.
              #   cssutils-0.9.8a1-py2.6.egg/cssutils/css/cssstyledeclaration.py:598: DeprecationWarning: Call to deprecated method '_setCSSValue'. Use ``property.propertyValue`` instead.
              view[element].removeProperty(p.name)
              view[element].setProperty(p.name, p.value, p.priority)
  return view

#------------------------------------------------------------------------------
# TODO: move this to use ET elements to remove the XML roundtrip...
# TODO: this currently strips out the <!DOCTYPE> line if it appears, eg:
#       <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
def inlineHtmlStyling_minidom(doc, css):
  def scb(element):
    if element.hasAttribute('style'):
      cssText = element.getAttribute('style')
      return cssutils.css.CSSStyleDeclaration(cssText=cssText)
    return None
  view = getHtmlStyleView(doc, css, styleCallback=scb)
  for element, style in view.items():
    v = style.getCssText(separator=u'')
    element.setAttribute('style', v)
  return doc
def inlineHtmlStyling(etdoc, css):
  html = serializeHtml(etdoc)
  doc  = minidom.parseString(html)
  ret  = inlineHtmlStyling_minidom(doc, css)
  return parseXml(ret.toxml('UTF-8'))

#------------------------------------------------------------------------------
def parseXml(data):
  src = StringIO(data)
  parser = ET.XMLParser()
  parser.parser.UseForeignDTD(True)
  # todo: allow some mechanism for the caller to provide additional entities
  parser.entity['nbsp']  = unichr(160)
  parser.entity['mdash'] = unichr(8212)
  parser.entity['copy']  = unichr(169)
  parser.entity['reg']   = unichr(174)
  etree = ET.ElementTree()
  tree = etree.parse(src, parser=parser)
  return tree

#------------------------------------------------------------------------------
def serializeHtml(xdoc):
  '''
  Serializes ElementTree `xdoc` to HTML.
  NOTE: `xdoc` is mutated to qualify any non-qualified attributes
  and elements with the HTML namespace.
  '''
  # note: qualifying all attributes and elements so that this error is
  # avoided:
  #   ValueError: cannot use non-qualified names with default_namespace option
  # IMHO, it should not be throwing this...
  # http://bugs.python.org/issue17088
  for node in xdoc.iter():
    if not node.tag.startswith('{'):
      node.tag = '{%s}%s' % (htmlns, node.tag)
    for attr in list(node.keys()):
      if not attr.startswith('{'):
        node.set('{%s}%s' % (htmlns, attr), node.get(attr))
        del node.attrib[attr]
  dst = StringIO()
  etree = ET.ElementTree(xdoc)
  etree.write(dst, default_namespace=htmlns)
  # todo: it would be nice if the HTML output were 'pretty-printed'...
  return dst.getvalue()

#------------------------------------------------------------------------------
# shamelessly scrubbed from:
#   http://www.w3schools.com/tags/ref_symbols.asp
# with additions for &copy; and &reg;
r2a_map = dict((

  (u'\u00a9', '(C)'),                  # © &#169; &copy; copyright symbol
  (u'\u00ae', '(R)'),                  # ® &#174; &reg; registered trademark

  # math symbols supported by HTML
  (u'\u2200', '<for-all>'),            # ∀ &#8704; &forall; for all
  (u'\u2202', '<part>'),               # ∂ &#8706; &part; part
  (u'\u2203', '<exists>'),             # ∃ &#8707; &exist; exists
  (u'\u2205', '<empty>'),              # ∅ &#8709; &empty; empty
  (u'\u2207', '<nabla>'),              # ∇ &#8711; &nabla; nabla
  (u'\u2208', '<isin>'),               # ∈ &#8712; &isin; isin
  (u'\u2209', '<notin>'),              # ∉ &#8713; &notin; notin
  (u'\u220b', '<ni>'),                 # ∋ &#8715; &ni; ni
  (u'\u220f', '<prod>'),               # ∏ &#8719; &prod; prod
  (u'\u2211', '<sum>'),                # ∑ &#8721; &sum; sum
  (u'\u2212', '<minus>'),              # − &#8722; &minus; minus
  (u'\u2217', '<lowast>'),             # ∗ &#8727; &lowast; lowast
  (u'\u221a', '<square-root>'),        # √ &#8730; &radic; square root
  (u'\u221d', '<proportional-to>'),    # ∝ &#8733; &prop; proportional to
  (u'\u221e', '<infinity>'),           # ∞ &#8734; &infin; infinity
  (u'\u2220', '<angle>'),              # ∠ &#8736; &ang; angle
  (u'\u2227', '<and>'),                # ∧ &#8743; &and; and
  (u'\u2228', '<or>'),                 # ∨ &#8744; &or; or
  (u'\u2229', '<cap>'),                # ∩ &#8745; &cap; cap
  (u'\u222a', '<cup>'),                # ∪ &#8746; &cup; cup
  (u'\u222b', '<integral>'),           # ∫ &#8747; &int; integral
  (u'\u2234', '<therefore>'),          # ∴ &#8756; &there4; therefore
  (u'\u223c', '<similar-to>'),         # ∼ &#8764; &sim; similar to
  (u'\u2245', '<congruent-to>'),       # ≅ &#8773; &cong; congruent to
  (u'\u2248', '<almost-equal>'),       # ≈ &#8776; &asymp; almost equal
  (u'\u2260', '<not-equal>'),          # ≠ &#8800; &ne; not equal
  (u'\u2261', '<equivalent>'),         # ≡ &#8801; &equiv; equivalent
  (u'\u2264', '<less-or-equal>'),      # ≤ &#8804; &le; less or equal
  (u'\u2265', '<greater-or-equal>'),   # ≥ &#8805; &ge; greater or equal
  (u'\u2282', '<subset-of>'),          # ⊂ &#8834; &sub; subset of
  (u'\u2283', '<superset-of>'),        # ⊃ &#8835; &sup; superset of
  (u'\u2284', '<not-subset-of>'),      # ⊄ &#8836; &nsub; not subset of
  (u'\u2286', '<subset-or-equal>'),    # ⊆ &#8838; &sube; subset or equal
  (u'\u2287', '<superset-or-equal>'),  # ⊇ &#8839; &supe; superset or equal
  (u'\u2295', '<circled-plus>'),       # ⊕ &#8853; &oplus; circled plus
  (u'\u2297', '<circled-times>'),      # ⊗ &#8855; &otimes; circled times
  (u'\u22a5', '<perpendicular>'),      # ⊥ &#8869; &perp; perpendicular
  (u'\u22c5', '<dot-operator>'),       # ⋅ &#8901; &sdot; dot operator

  # greek letters supported by HTML
  (u'\u0391', '<Alpha>'),              # Α &#913; &Alpha; Alpha
  (u'\u0392', '<Beta>'),               # Β &#914; &Beta; Beta
  (u'\u0393', '<Gamma>'),              # Γ &#915; &Gamma; Gamma
  (u'\u0394', '<Delta>'),              # Δ &#916; &Delta; Delta
  (u'\u0395', '<Epsilon>'),            # Ε &#917; &Epsilon; Epsilon
  (u'\u0396', '<Zeta>'),               # Ζ &#918; &Zeta; Zeta
  (u'\u0397', '<Eta>'),                # Η &#919; &Eta; Eta
  (u'\u0398', '<Theta>'),              # Θ &#920; &Theta; Theta
  (u'\u0399', '<Iota>'),               # Ι &#921; &Iota; Iota
  (u'\u039a', '<Kappa>'),              # Κ &#922; &Kappa; Kappa
  (u'\u039b', '<Lambda>'),             # Λ &#923; &Lambda; Lambda
  (u'\u039c', '<Mu>'),                 # Μ &#924; &Mu; Mu
  (u'\u039d', '<Nu>'),                 # Ν &#925; &Nu; Nu
  (u'\u039e', '<Xi>'),                 # Ξ &#926; &Xi; Xi
  (u'\u039f', '<Omicron>'),            # Ο &#927; &Omicron; Omicron
  (u'\u03a0', '<Pi>'),                 # Π &#928; &Pi; Pi
  (u'\u03a1', '<Rho>'),                # Ρ &#929; &Rho; Rho
  (u'\u03a3', '<Sigma>'),              # Σ &#931; &Sigma; Sigma
  (u'\u03a4', '<Tau>'),                # Τ &#932; &Tau; Tau
  (u'\u03a5', '<Upsilon>'),            # Υ &#933; &Upsilon; Upsilon
  (u'\u03a6', '<Phi>'),                # Φ &#934; &Phi; Phi
  (u'\u03a7', '<Chi>'),                # Χ &#935; &Chi; Chi
  (u'\u03a8', '<Psi>'),                # Ψ &#936; &Psi; Psi
  (u'\u03a9', '<Omega>'),              # Ω &#937; &Omega; Omega
  (u'\u03b1', '<alpha>'),              # α &#945; &alpha; alpha
  (u'\u03b2', '<beta>'),               # β &#946; &beta; beta
  (u'\u03b3', '<gamma>'),              # γ &#947; &gamma; gamma
  (u'\u03b4', '<delta>'),              # δ &#948; &delta; delta
  (u'\u03b5', '<epsilon>'),            # ε &#949; &epsilon; epsilon
  (u'\u03b6', '<zeta>'),               # ζ &#950; &zeta; zeta
  (u'\u03b7', '<eta>'),                # η &#951; &eta; eta
  (u'\u03b8', '<theta>'),              # θ &#952; &theta; theta
  (u'\u03b9', '<iota>'),               # ι &#953; &iota; iota
  (u'\u03ba', '<kappa>'),              # κ &#954; &kappa; kappa
  (u'\u03bb', '<lambda>'),             # λ &#955; &lambda; lambda
  (u'\u03bc', '<mu>'),                 # μ &#956; &mu; mu
  (u'\u03bd', '<nu>'),                 # ν &#957; &nu; nu
  (u'\u03be', '<xi>'),                 # ξ &#958; &xi; xi
  (u'\u03bf', '<omicron>'),            # ο &#959; &omicron; omicron
  (u'\u03c0', '<pi>'),                 # π &#960; &pi; pi
  (u'\u03c1', '<rho>'),                # ρ &#961; &rho; rho
  (u'\u03c2', '<sigmaf>'),             # ς &#962; &sigmaf; sigmaf
  (u'\u03c3', '<sigma>'),              # σ &#963; &sigma; sigma
  (u'\u03c4', '<tau>'),                # τ &#964; &tau; tau
  (u'\u03c5', '<upsilon>'),            # υ &#965; &upsilon; upsilon
  (u'\u03c6', '<phi>'),                # φ &#966; &phi; phi
  (u'\u03c7', '<chi>'),                # χ &#967; &chi; chi
  (u'\u03c8', '<psi>'),                # ψ &#968; &psi; psi
  (u'\u03c9', '<omega>'),              # ω &#969; &omega; omega
  (u'\u03d1', '<theta-symbol>'),       # ϑ &#977; &thetasym; theta symbol
  (u'\u03d2', '<upsilon-symbol>'),     # ϒ &#978; &upsih; upsilon symbol
  (u'\u03d6', '<pi-symbol>'),          # ϖ &#982; &piv; pi symbol

  # other symbols supported by HTML
  (u'\u0152', 'OE'),                   # Œ &#338; &OElig; capital ligature OE
  (u'\u0153', 'oe'),                   # œ &#339; &oelig; small ligature oe
  # TODO: should these be decorated?...
  (u'\u0160', 'S'),                    # Š &#352; &Scaron; capital S with caron
  (u'\u0161', 's'),                    # š &#353; &scaron; small S with caron
  (u'\u0178', 'Y'),                    # Ÿ &#376; &Yuml; capital Y with diaeres
  (u'\u0192', '?'),                    # ƒ &#402; &fnof; f with hook

  (u'\u02c6', '^'),                    # ˆ &#710; &circ; modifier letter circumflex accent
  (u'\u02dc', '~'),                    # ˜ &#732; &tilde; small tilde
  (u'\u2002', ' '),                    #   &#8194; &ensp; en space
  (u'\u2003', ' '),                    #   &#8195; &emsp; em space
  (u'\u2009', ' '),                    #   &#8201; &thinsp; thin space
  (u'\u200c', '?'),                    # ‌ &#8204; &zwnj; zero width non-joiner
  (u'\u200d', '?'),                    # ‍ &#8205; &zwj; zero width joiner
  (u'\u200e', '?'),                    # ‎ &#8206; &lrm; left-to-right mark
  (u'\u200f', '?'),                    # ‏ &#8207; &rlm; right-to-left mark
  (u'\u2013', '-'),                    # – &#8211; &ndash; en dash
  (u'\u2014', '-'),                    # — &#8212; &mdash; em dash
  (u'\u2018', '\''),                   # ‘ &#8216; &lsquo; left single quotation mark
  (u'\u2019', '\''),                   # ’ &#8217; &rsquo; right single quotation mark
  (u'\u201a', '\''),                   # ‚ &#8218; &sbquo; single low-9 quotation mark
  (u'\u201c', '"'),                    # “ &#8220; &ldquo; left double quotation mark
  (u'\u201d', '"'),                    # ” &#8221; &rdquo; right double quotation mark
  (u'\u201e', '"'),                    # „ &#8222; &bdquo; double low-9 quotation mark
  (u'\u2020', '?'),                    # † &#8224; &dagger; dagger
  (u'\u2021', '?'),                    # ‡ &#8225; &Dagger; double dagger
  (u'\u2022', '*'),                    # • &#8226; &bull; bullet
  (u'\u2026', '...'),                  # … &#8230; &hellip; horizontal ellipsis
  (u'\u2030', '0/00'),                 # ‰ &#8240; &permil; per mille
  (u'\u2032', '\''),                   # ′ &#8242; &prime; minutes
  (u'\u2033', '"'),                    # ″ &#8243; &Prime; seconds
  (u'\u2039', '<'),                    # ‹ &#8249; &lsaquo; single left angle quotation
  (u'\u203a', '>'),                    # › &#8250; &rsaquo; single right angle quotation
  (u'\u203e', '-'),                    # ‾ &#8254; &oline; overline
  (u'\u20ac', '<euro>'),               # € &#8364; &euro; euro
  (u'\u2122', '<TM>'),                 # ™ &#8482; &trade; trademark
  (u'\u2190', '<-'),                   # ← &#8592; &larr; left arrow
  (u'\u2191', '?'),                    # ↑ &#8593; &uarr; up arrow
  (u'\u2192', '->'),                   # → &#8594; &rarr; right arrow
  (u'\u2193', '?'),                    # ↓ &#8595; &darr; down arrow

  # additional right arrows scrubbed from:
  #   http://right-arrow.net/
  (u'\u21d2', '=>'),                   # ⇒ &#8658; &rArr;
  (u'\u21ac', '->'),                   # ↬ &#8620;
  (u'\u21c9', '=>'),                   # ⇉ &#8649;
  (u'\u21e8', '=>'),                   # ⇨ &#8680;
  (u'\u21e5', '->|'),                  # ⇥ &#8677;
  (u'\u21e2', '->'),                   # ⇢ &#8674;
  (u'\u21aa', '->'),                   # ↪ &#8618;
  (u'\u21a6', '|->'),                  # ↦ &#8614;
  (u'\u21f6', '=>'),                   # ⇶ &#8694;
  (u'\u21a3', '>->'),                  # ↣ &#8611;
  (u'\u21f0', '|=>'),                  # ⇰ &#8688;
  (u'\u219d', '~>'),                   # ↝ &#8605;
  (u'\u21a0', '->>'),                  # ↠ &#8608;
  (u'\u21fe', '->'),                   # ⇾ &#8702;

  # others...
  (u'\u21e6', '<='),                   # ⇦ &#8678;

  ))

def reduce2ascii(text):
  ret = []
  for c in text:
    if ord(c) < 128:
      ret.append(c)
      continue
    # todo: handle ascii chars in the 128 -> 255 range...
    ret.append(r2a_map.get(c, '?'))
  return ''.join(ret)
  #return text.replace(u'\u21d2', '=>')

#------------------------------------------------------------------------------
_removeCidsRe = re.compile('\\!\\[([^\\]]*)\\]\\(cid:[^)]*\\)')
def removeCids(text):
  return _removeCidsRe.sub('[\\1]', text)

#------------------------------------------------------------------------------
# todo: move this?
smtp_header_element_upper = ['cc', 'id', 'spf']
def smtpHeaderFormat(h):
  return '-'.join([e.upper() if e in smtp_header_element_upper else e.title()
                   for e in [e.lower() for e in h.split('-')]])

#------------------------------------------------------------------------------
# todo: installing a global unicode decode/encode translation error handler...
#       this is not ideal... see Email.getSubject() for details...
#------------------------------------------------------------------------------
def genemail_unicode2ascii(err):
  if err.reason != 'ordinal not in range(128)':
    raise err
  # todo: shouldn't this just use `r2a_map`?...
  if err.object[err.start] == unichr(160):
    # &nbsp;
    return (u' ', err.start + 1)
  if err.object[err.start] == unichr(8212):
    # &mdash;
    return (u'--', err.start + 1)
  return (u'?', err.start + 1)
#------------------------------------------------------------------------------
try:
  codecs.lookup_error('genemail_unicode2ascii')
except LookupError:
  codecs.register_error('genemail_unicode2ascii', genemail_unicode2ascii)

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# auth: Philip J Grabner <grabner@cadit.com>
# date: 2013/07/03
# copy: (C) Copyright 2013 Cadit Inc., see LICENSE.txt
#------------------------------------------------------------------------------

import os, sys, re
from setuptools import setup, find_packages

# require python 2.7+
assert(sys.version_info[0] > 2
       or sys.version_info[0] == 2
       and sys.version_info[1] >= 7)

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()

test_requires = [
  'nose                 >= 1.2.1',
  'coverage             >= 3.5.3',
  ]

requires = [
  'TemplateAlchemy      >= 0.1.14',
  'cssutils             >= 0.9.10b1',
  'cssselect            >= 0.7.1',
  'py-dom-xpath         >= 0.1',
  'html2text            >= 3.200.3',
  ]

entrypoints = {
  'console_scripts': [
    'genemail           = genemail.cli:main',
    ],
  }

setup(
  name                  = 'genemail',
  version               = '0.1.2',
  description           = 'A templated email generation library',
  long_description      = README,
  classifiers           = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Programming Language :: Python',
    'Operating System :: OS Independent',
    'Topic :: Software Development',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'License :: Public Domain',
    ],
  author                = 'Philip J Grabner, Cadit Health Inc',
  author_email          = 'oss@cadit.com',
  url                   = 'http://github.com/cadithealth/genemail',
  keywords              = 'template email generation html text smtp',
  packages              = find_packages(),
  namespace_packages    = ['genemail'],
  include_package_data  = True,
  zip_safe              = True,
  install_requires      = requires,
  tests_require         = test_requires,
  test_suite            = 'genemail',
  entry_points          = entrypoints,
  license               = 'MIT (http://opensource.org/licenses/MIT)',
  )

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
#------------------------------------------------------------------------------
# file: $Id$
# lib:  genemail.manager
# auth: Philip J Grabner <grabner@cadit.com>
# date: 2013/07/09
# copy: (C) Copyright 2013 Cadit Health Inc., All Rights Reserved.
#------------------------------------------------------------------------------

from __future__ import absolute_import

__all__ = ('Manager',)

from templatealchemy.util import adict, callingPkgName
from . import sender as sendermod
from . import email

#------------------------------------------------------------------------------
class Manager(object):

  #----------------------------------------------------------------------------
  def __init__(self, provider=None, sender=None):
    '''
    A ``genemail.manager.Manager`` object is the main clearinghouse
    for generating templatized emails. The main objective is that it
    provides access to :class:`genemail.email.Email` objects.

    :Parameters:

    provider : TemplateAlchemy.Template, optional
      the template provider; if not specified, defaults to loading a
      `pkg` provider in the calling package's namespace.

    sender : :class:`genemail.sender.Sender`, optional
      the email sending agent; if not specified, defaults to a
      SmtpSender with default parameters.
    '''
    self.provider = provider
    if not self.provider:
      self.provider = TA.Template(
        source='pkg:%s:' % (callingPkgName(ignore='genemail'),))
    self.sender   = sender or sendermod.SmtpSender()
    self.default  = email.Email(self, None)

  #----------------------------------------------------------------------------
  def newEmail(self, name=None):
    return email.Email(self, name, default=self.default)

#------------------------------------------------------------------------------
# end of $Id$
#------------------------------------------------------------------------------
